package de.hhu.ba.yoshikoWrapper.core;
//TODO: ADD LOGGER SYSTEM

import org.cytoscape.model.CyColumn;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.model.CyNode;
import org.cytoscape.work.AbstractTask;
import org.cytoscape.work.TaskMonitor;
import org.slf4j.Logger;

import de.hhu.ba.yoshikoWrapper.logging.YoshikoLogger;
import de.hhu.ba.yoshikoWrapper.swig.LibraryInterface;
import de.hhu.ba.yoshikoWrapper.swig.SWIGTYPE_p_std__vectorT_int_t;
import de.hhu.ba.yoshikoWrapper.swig.SWIGTYPE_p_yskInput__LibraryInput;
import de.hhu.ba.yoshikoWrapper.swig.SWIGTYPE_p_ysk__ClusterEditingSolutions;

public class AlgorithmTask extends AbstractTask {
	
	
	//Symbolic links
	private static Logger logger = YoshikoLogger.getInstance().getLogger();

	//Parameters
	private double insertionCostDefault;
	private double deletionCostDefault;
	private CyColumn permanentColumn;
	private CyColumn forbiddenColumn;
	private CyColumn weightColumn;
	private int timeLimit;
	private CyNetwork net;

	public AlgorithmTask(
			CyNetwork net,
			int timeLimit,
			CyColumn weightColumn,
			CyColumn permanentColumn,
			CyColumn forbiddenColumn,
			double insertionCostDefault,
			double deletionCostDefault
			) 
	{
		this.net = net;
		this.timeLimit = timeLimit;
		this.weightColumn = weightColumn;
		this.permanentColumn = permanentColumn;
		this.forbiddenColumn = forbiddenColumn;
		this.insertionCostDefault = insertionCostDefault;
		this.deletionCostDefault = deletionCostDefault;
	}

	@Override
	public void run(TaskMonitor taskMonitor) throws Exception {
		taskMonitor.setTitle(LocalizationManager.yoshikoStrings().getString("yoshTask"));
		taskMonitor.setProgress(0.0);
		//Get current network
		if (net == null) {
			logger.warn("CoreAlgorithm called on a net that is NULL!");
			return;
		}
		
		//Set time limit
		LibraryInterface.setTimeLimit(timeLimit);

		//Create a node map to identify nodes and eges in the solution
		NodeMap nodeMap  = new NodeMap(net);
		taskMonitor.setProgress(0.1);

		//Generate an input instance from the network
		SWIGTYPE_p_yskInput__LibraryInput input = NetworkParser.parseNetwork(
				net,
				nodeMap,
				weightColumn,
				permanentColumn, 
				forbiddenColumn,
				deletionCostDefault
				);
		taskMonitor.setProgress(0.2);

		//Set the default value for insertion cost
		LibraryInterface.LibraryInput_setDefaultInsertionCost(input, insertionCostDefault);

		//Call Yoshiko <<< Algorithm is performed here
		SWIGTYPE_p_ysk__ClusterEditingSolutions solutions = LibraryInterface.processLibraryInput(input);
		taskMonitor.setProgress(0.9);

		
		long numberOfSolutions = LibraryInterface.ClusterEditingSolutions_getNumberOfSolutions(solutions);
		System.out.println("Found: "+numberOfSolutions+" solutions!");
		double modificationCost = LibraryInterface.ClusterEditingSolutions_getTotalCost(solutions);
		System.out.println("Paid a total modification cost of: "+modificationCost);
		for (long i=0;i<numberOfSolutions;i++) {
			System.out.println("Processing solution "+(i+1)+" of "+numberOfSolutions);
			String columnName = "YOSHIKO_SOLUTION_"+(i+1);
			net.getDefaultNodeTable().deleteColumn(columnName);
			net.getDefaultNodeTable().createColumn(columnName, String.class, false);
			long numberOfClusters = LibraryInterface.ClusterEditingSolutions_getNumberOfClusters(solutions, i);
			for (long k=0;k<numberOfClusters;k++) {
				System.out.println("Processing cluster "+(k+1)+" of "+numberOfClusters);
				SWIGTYPE_p_std__vectorT_int_t cluster = LibraryInterface.ClusterEditingSolutions_getCluster(solutions, i, k);
				long sizeOfCluster = LibraryInterface.IntVector_size(cluster);
				for (int l=0;l<sizeOfCluster;l++) { //Unsafe mismatch int long
					System.out.println("Processing entry "+(l+1)+ " of "+sizeOfCluster);
					int nodeID = LibraryInterface.IntVector_get(cluster, l);
					CyNode node = nodeMap.indexOf(nodeID); //<<< Another int/long conversion
					if (node == null) {
						System.out.println("FATAL ERROR: Node was not previously mapped!");
						return;
					}
					net.getRow(node).set(columnName, ""+(k+1));
				}
			}
		}
		
		LibraryInterface.delete_LibraryInput(input);		
		taskMonitor.setProgress(1.0);
	}
}
