package de.hhu.ba.yoshikoWrapper.gui;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JRadioButton;

import org.cytoscape.application.swing.CytoPanelComponent;
import org.cytoscape.application.swing.CytoPanelName;
import org.cytoscape.work.AbstractTask;
import org.cytoscape.work.TaskIterator;

import de.hhu.ba.yoshikoWrapper.core.AlgorithmTask;
import de.hhu.ba.yoshikoWrapper.core.CyCore;
import de.hhu.ba.yoshikoWrapper.core.YoshikoLoader;

/**This class describes the Swing Panel that the user interacts with in cytoscape
 * @author Philipp Spohr, Aug 6, 2017
 *
 */
@SuppressWarnings("serial")
public class MainPanel extends ComfortPanel implements CytoPanelComponent {

	//SWING COMPONENTS
	private final LanguageSwitcher langSwitcher;
	
	private final LibraryPanel libraryPanel;

	private final EditCostPanel ecPanel;
	
	private final ButtonGroup heuristicGroup;
	private final JRadioButton useHeuristic;
	private final JRadioButton useILP;
	
	private final TimeLimitSetter timeLimitSetter;
	
	private final ReductionRulesChooser reductionRulesChooser;
	
	private final JCheckBox useTriangleCutsBox;
	private final JCheckBox usePartitionCutsBox;
	
	//SYMBOLIC LINKS
	private final SolutionsPanel solutionsPanel;
	
	/**
	 * Main constructor, creates a new Panel and initializes subcomponents
	 * @param solPanel 
	 */
	public MainPanel(SolutionsPanel solPanel) {
		
		this.solutionsPanel = solPanel;
		
		//SWING INIT
		this.setLayout(new BoxLayout(this,BoxLayout.PAGE_AXIS));
		
		//Initialize Swing components
		langSwitcher = new LanguageSwitcher();
		libraryPanel = new LibraryPanel();
		
		ecPanel = new EditCostPanel();
		
		heuristicGroup = new ButtonGroup();
		useILP = new JRadioButton("Use Integer Linear Programming");
		useILP.setSelected(true);
		useHeuristic = new JRadioButton("Use Heuristic");
		heuristicGroup.add(useILP);
		heuristicGroup.add(useHeuristic);
		

		JButton runButton = new JButton("RUN");
		runButton.addActionListener(new ActionListener() {

			
			@Override
			public void actionPerformed(ActionEvent e) {
				if (YoshikoLoader.isLibraryLoaded()){
					AbstractTask yoshiko = new AlgorithmTask(
							CyCore.cy.getCurrentNetwork(),
							timeLimitSetter.getTimeLimit(), 
							ecPanel.getWeightColumn(),
							ecPanel.getPermanentColumn(),
							ecPanel.getForbiddenColumn(),
							ecPanel.getDefaultInsertionCost(),
							ecPanel.getDefaultDeletionCost()
							);
					CyCore.dialogTaskManager.execute(new TaskIterator(1,yoshiko));
					solutionsPanel.setVisible(true);
				}
			}
			
		});

		reductionRulesChooser = new ReductionRulesChooser();
		
		timeLimitSetter = new TimeLimitSetter();
		
		useTriangleCutsBox = new JCheckBox("Use Triangle Cuts");
		usePartitionCutsBox = new JCheckBox("Use Partition Cuts");

		
		this.addAll(
				langSwitcher,
				libraryPanel,
				ecPanel,
				useILP,
				useHeuristic,
				timeLimitSetter,
				reductionRulesChooser,
				useTriangleCutsBox,
				usePartitionCutsBox,
				runButton
				);
		
		this.setVisible(true);
	}

	
	
	//GETTER / SETTER

	public Component getComponent() {
		return this;
	}


	/* (non-Javadoc)
	 * @see org.cytoscape.application.swing.CytoPanelComponent#getCytoPanelName()
	 */
	public CytoPanelName getCytoPanelName() {
		//By convention most plugins that provide a "toolbox"-like interface use the WEST orientation
		return CytoPanelName.WEST;
	}

	public String getTitle() {
		//TODO: Be creative I guess
		return "Yoshiko";
	}


	public Icon getIcon() {
		//TODO: 
		return null;
	}
}
